@@ -53,7 +53,7 @@ class Circle(sge.Object):
             self.image_yscale = 2
 
     def event_step(self, time_passed, delta_mult):
-        self.image_rotation += 2 * delta_mult
+        self.image_rotation += delta_mult
         sge.game.current_room.project_circle(self.x, self.y, self.z + 1, 8,
                                              outline=sge.Color("green"))
 
