@@ -27,10 +27,11 @@ import sge
 from sge import r
 from sge.r import (_get_dot_sprite, _get_line_sprite, _get_rectangle_sprite,
                    _get_ellipse_sprite, _get_circle_sprite,
-                   _get_polygon_sprite, o_update_collision_lists,
-                   r_update_fade, r_update_dissolve, r_update_pixelate,
-                   r_update_wipe_left, r_update_wipe_right, r_update_wipe_up,
-                   r_update_wipe_down, r_update_wipe_upleft,
+                   _get_polygon_sprite, o_update_object_areas,
+                   o_update_collision_lists, r_get_rectangle_object_areas,
+                   r_set_object_areas, r_update_fade, r_update_dissolve,
+                   r_update_pixelate, r_update_wipe_left, r_update_wipe_right,
+                   r_update_wipe_up, r_update_wipe_down, r_update_wipe_upleft,
                    r_update_wipe_upright, r_update_wipe_downleft,
                    r_update_wipe_downright, r_update_wipe_matrix,
                    r_update_iris_in, r_update_iris_out, s_get_image)
@@ -72,6 +73,20 @@ class Room(object):
 
        The vertical position of the background in the room.
 
+    .. attribute:: object_area_width
+
+       The width of this room's object areas in pixels.  If set to
+       :const:`None`, :attr:`sge.game.width` is used.  For optimum
+       performance, this should generally be about the average width of
+       objects in the room which check for collisions.
+
+    .. attribute:: object_area_height
+
+       The height of this room's object areas in pixels.  If set to
+       :const:`None`, :attr:`sge.game.height` is used.  For optimum
+       performance, this should generally be about the average height of
+       objects in the room which check for collisions.
+
     .. attribute:: alarms
 
        A dictionary containing the alarms of the room.  Each value
@@ -85,13 +100,84 @@ class Room(object):
        A list containing all :class:`sge.Object` objects in the
        room.  (Read-only)
 
+    .. attribute:: object_areas
+
+       A 2-dimensional list of object areas, indexed in the following
+       way::
+
+           object_areas[x][y]
+
+       Where ``x`` is the horizontal location of the left edge of the
+       area in the room divided by :attr:`object_area_width`, and ``y``
+       is the vertical location of the top edge of the area in the room
+       divided by :attr:`object_area_height`.
+
+       For example, if :attr:`object_area_width` is ``32`` and
+       :attr:`object_area_height` is ``48``, then
+       ``object_areas[2][4]`` indicates the object area with an x
+       location of 64 and a y location of 192.
+
+       Each object area is a set containing :class:`sge.Object` objects
+       whose sprites or bounding boxes reside within the object area.
+
+       Object areas are only created within the room, i.e. the
+       horizontal location of an object area will always be less than
+       :attr:`width`, and the vertical location of an object area will
+       always be less than :attr:`height`.  Depending on the size of
+       collision areas and the size of the room, however, the last row
+       and/or the last column of collision areas may partially reside
+       outside of the room.
+
+       .. note::
+
+          It is generally easier to use :meth:`sge.Room.get_objects_at`
+          than to access this list directly.
+
+    .. attribute:: object_area_void
+
+       A set containing :class:`sge.Object` objects whose sprites or
+       bounding boxes reside within any area not covered by the room's
+       object area.
+
+       .. note::
+
+          Depending on the size of object areas and the size of the
+          room, the "void" area may not include the entirety of the
+          outside of the room.  There may be some space to the right of
+          and/or below the room which is covered by collision areas.
+
     .. attribute:: rd
 
        Reserved dictionary for internal use by the SGE.  (Read-only)
     """
 
+    @property
+    def object_area_width(self):
+        return self.__object_area_width
+
+    @object_area_width.setter
+    def object_area_width(self, value):
+        if value is None:
+            value = sge.game.width
+
+        self.__object_area_width = value
+        r_set_object_areas(self)
+
+    @property
+    def object_area_height(self):
+        return self.__object_area_height
+
+    @object_area_height.setter
+    def object_area_height(self, value):
+        if value is None:
+            value = sge.game.height
+
+        self.__object_area_height = value
+        r_set_object_areas(self)
+
     def __init__(self, objects=(), width=None, height=None, views=None,
-                 background=None, background_x=0, background_y=0):
+                 background=None, background_x=0, background_y=0,
+                 object_area_width=None, object_area_height=None):
         """
         Arguments:
 
@@ -112,6 +198,14 @@ class Room(object):
         self.height = height if height is not None else sge.game.height
         self.rd["swidth"] = self.width
         self.rd["sheight"] = self.height
+
+        if object_area_width is None:
+            object_area_width = sge.game.width
+        if object_area_height is None:
+            object_area_height = sge.game.height
+
+        self.__object_area_width = object_area_width
+        self.__object_area_height = object_area_height
         self.background_x = background_x
         self.background_y = background_y
         self.alarms = {}
@@ -131,29 +225,12 @@ class Room(object):
         self.rd["started"] = False
 
         self.objects = []
+        r_set_object_areas(self)
 
         self.add(sge.game.mouse)
         for obj in objects:
             self.add(obj)
 
-        if self.views:
-            size = max(int(self.views[0].width / 10),
-                       int(self.views[0].height / 10))
-        else:
-            size = sge.COLLISION_AREA_SIZE_DEFAULT
-
-        self.rd["collision_area_size"] = size
-        self.rd["collision_areas"] = []
-        for i in six.moves.range(0, self.width, size):
-            column = [[] for j in six.moves.range(0, self.height, size)]
-            self.rd["collision_areas"].append(column)
-
-        # The "Void" is the area outside the room.  This area is
-        # infinite, so everything in the Void is checked against
-        # everything else in the Void (otherwise an infinite grid, which
-        # is impossible, would be needed).
-        self.rd["collision_area_void"] = []
-
     def add(self, obj):
         """
         Add an object to the room.
@@ -169,8 +246,11 @@ class Room(object):
 
             if self is sge.game.current_room and self.rd["started"]:
                 obj.event_create()
-                obj.rd["collision_areas"] = []
+                obj.rd["object_areas"] = set()
+                o_update_object_areas(obj)
                 o_update_collision_lists(obj)
+                if obj.active:
+                    r._active_objects.add(obj)
             else:
                 self.__new_objects.append(obj)
 
@@ -188,8 +268,10 @@ class Room(object):
         while obj in self.__new_objects:
             self.__new_objects.remove(obj)
 
-        o_update_collision_lists(obj)
         if self is sge.game.current_room:
+            o_update_object_areas(obj)
+            o_update_collision_lists(obj)
+            r._active_objects.discard(obj)
             obj.event_destroy()
 
     def start(self, transition=None, transition_time=1500,
@@ -264,10 +346,15 @@ class Room(object):
 
         r._colliders = []
         r._collision_checkers = []
+        r._active_objects = set()
 
+        r_set_object_areas(self)
         for obj in self.objects:
-            obj.rd["collision_areas"] = []
+            obj.rd["object_areas"] = set()
+            o_update_object_areas(obj)
             o_update_collision_lists(obj)
+            if obj.active:
+                r._active_objects.add(obj)
 
         # This is stored in a variable to prevent problems with
         # rd["started"] being False during the start/create events.
@@ -285,6 +372,38 @@ class Room(object):
         # due to delta timing, and make sure transitions happen fully.
         r.game_clock.tick()
 
+    def get_objects_at(self, x, y, width, height):
+        """
+        Return a set of objects near a particular area.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the room of the
+          left edge of the area.
+        - ``y`` -- The vertical location relative to the room of the
+          top edge of the area.
+        - ``width`` -- The width of the area in pixels.
+        - ``height`` -- The height of the area in pixels.
+
+        .. note::
+
+           This function does not ensure that objects returned are
+           actually *within* the given area.  It simply combines all
+           object areas that need to be checked into a single set.  To
+           ensure that an object is actually within the area, you must
+           check the object manually, or use
+           :func:`sge.collision.rectangle` instead.
+        """
+        area = set()
+        for a in r_get_rectangle_object_areas(self, x, y, width, height):
+            if a is None:
+                area |= self.object_area_void
+            else:
+                area |= self.object_areas[a[0]][a[1]]
+
+        return area
+            
+
     def project_dot(self, x, y, z, color):
         """
         Project a single-pixel dot onto the room.
